import requests
from flask import Flask
from flask import request, abort
from werkzeug.serving import make_server
from threading import Thread
import functools
from termcolor import colored

class Endpoint():
    def __init__(self, type, func):
        self.type = type
        self.func = func

class Request():
    def __init__(self, json:dict, ws = None):
        self.json = json
        self.ws = ws

class Response():
    def __init__(self, status:int, json:dict):
        self.json = json
        self.status = status

class Potassium():
    def __init__(self, name):
        def default_func():
            return
        self.name = name
        self.init_func = default_func
        self.endpoints = {} # dictionary to store unlimited Endpoints, by unique route
        self.context = {}

    # init runs once on server start
    def init(self, func): 
        def wrapper():
            self.context = func()
        self.init_func=wrapper
        return wrapper
    
    # handler is a blocking http POST handler
    def handler(self, route = "/"):
        def actual_decorator(func):
            @functools.wraps(func)
            def wrapper(request):
                # send in app's stateful context, and the request
                return func(self.context, request)
            self.endpoints[route] = Endpoint(type="handler", func=wrapper)
            return wrapper
        return actual_decorator
    
    # async_handler is a non-blocking http POST handler
    def async_handler(self, route: str = "/"):
        def actual_decorator(func):
            @functools.wraps(func)
            def wrapper(request):
                # send in app's stateful context, and the request
                return func(self.context, request)
            self.endpoints[route] = Endpoint(type="async_handler", func=wrapper)
            return wrapper
        return actual_decorator
    
    # optional util
    # set_context sets the app's context. This overwrites the prior context dictionary
    def set_context(self, val:dict):
        self.context = val

    # optional util
    # get_context gets the app's context
    def get_context(self):
        return self.context
    
    def _create_flask_app(self):
        flask_app = Flask(__name__)

        # ingest into single endpoint and spread out to multiple downstream funcs
        @flask_app.route('/', defaults={'path': ''}, methods=["POST"])
        @flask_app.route('/<path:path>', methods=["POST"])
        def handle(path):
            route = "/" + path
            if route not in self.endpoints:
                abort(404)

            endpoint = self.endpoints[route]

            if endpoint.type == "handler":
                req = Request(
                    json = request.get_json()
                )
                return endpoint.func(req).json
            
            if endpoint.type == "async_handler":
                req = Request(
                    json = request.get_json()
                )
                # run as threaded task
                def task(func, req):
                    response = func(req)
                    # we currently do nothing with the response
                thread = Thread(target=task, args=(endpoint.func, req))
                thread.start()

                # send task start success message
                return {"started": True}
            
        return flask_app

    # serve runs the http server
    def serve(self, port = 8000):
        print(colored("------\nStarting Potassium Server 🍌", 'yellow'))   
        print(colored("Running init()", 'yellow'))
        self.init_func()
        flask_app = self._create_flask_app()
        server = make_server('localhost', port, flask_app)
        print(colored("Serving at http://localhost:8000\n------", 'green'))
        server.serve_forever()


from distutils.core import setup
import setuptools
from pathlib import Path

this_directory = Path(__file__).parent
long_description = (this_directory / "README.md").read_text()

setup(
    name='potassium',
    packages=['potassium'],
    version='0.0.7',
    license='Apache License 2.0',
    # Give a short description about your library
    description='The potassium package is a flask-like HTTP server for serving large AI models',
    long_description=long_description,
    long_description_content_type='text/markdown',
    author='Erik Dunteman',
    author_email='erik@banana.dev',
    url='https://www.banana.dev',
    keywords=['Banana server', 'HTTP server', 'Banana', 'Framework'],
    setup_requires=['wheel'],
    install_requires=[
        "Flask",
        "requests",
        "termcolor"
    ],
    classifiers=[
        # Chose either "3 - Alpha", "4 - Beta" or "5 - Production/Stable" as the current state of your package
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Build Tools',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
    ],
)



import requests

def send_webhook(url: str, json: dict):
    try:
        res = requests.post(url, json=json)
    except requests.exceptions.ConnectionError:
        print(f"Webhook to {url} failed with connection error")